// Inspired by
// https://observablehq.com/@d3/parallel-coordinates
// https://observablehq.com/@d3/brushable-parallel-coordinates

import {
  select,
  transition,
  scaleLinear,
  scalePoint,
  scaleOrdinal,
  schemeCategory10,
  extent,
  line,
  brushY,
} from 'd3';
import { observeResize } from '@curran/responsive-axes';
import { parallelCoordinates } from './parallelCoordinates';
import { languages } from './languages-gallery';

// alert(JSON.stringify(data));
// alert(JSON.stringify(languages));

// Assign d.id to the index.
// This is used later for object constancy in transitions.
languages.forEach((d, i) => {
  d.id = i;
});

const columns = [
  'code', //: "fra",
  // 'iso', //: "fra",
  // 'isoLabel', //: "français",
  'itemLabel', //: "French",
  // 'language', //: "Q21",
  // 'languageLabel', //: "French",
  // 'languageLabelNative', //: "français",
  // 'languageLabelsCount', //: 1,
  'population', //: 208157220,
  // 'populationQualifier', //: "10-100M",
  'records', //: 381147,
  'recordsFemale', //: 24397,
  'recordsMale', //: 353155,
  'recordsOthers', //: 3595,
  'speakers', //: 539,
  'speakersFemales', //: 212,
  'speakersMales', //: 296,
  'speakersOthers', //: 31,
  'percent', //: 60.8,
  // 'wiki', //: "fr",
  // 'wikidata', //: "Q150",
  'words', //: 231601
];
const columnTypes = {
  code: 'categorical',
  // iso: 'categorical',
  // isoLabel: 'categorical',
  itemLabel: 'categorical',
  // language: 'categorical',
  // languageLabel: 'categorical',
  // languageLabelNative: 'categorical',
  // languageLabelsCount: 'quantitative',
  percent: 'quantitative',
  population: 'quantitative',
  // populationQualifier: 'categorical',
  records: 'quantitative',
  recordsFemale: 'quantitative',
  recordsMale: 'quantitative',
  recordsOthers: 'quantitative',
  speakers: 'quantitative',
  speakersFemales: 'quantitative',
  speakersMales: 'quantitative',
  speakersOthers: 'quantitative',
  // wiki: 'categorical',
  // wikidata: 'categorical',
  words: 'quantitative',
};

const colorValue = (d) => d.code;
const idValue = (d) => d.id;

export const main = (container, { state, setState }) => {
  // Initialize dimensions by observing resise.
  const dimensions = observeResize({
    state,
    setState,
    container,
  });
  if (dimensions === null) return;
  const { width, height } = dimensions;

  // Initialize the brushed intervals state to an empty object.
  if (!state.brushedIntervals) {
    setState((state) => ({
      ...state,
      brushedIntervals: {},
    }));
    return;
  }
  const { brushedIntervals } = state;

  // Updates the brushed intervals when brushes change.
  const updateBrushedInterval = ({ column, interval }) => {
    setState((state) => ({
      ...state,
      brushedIntervals: {
        ...state.brushedIntervals,
        [column]: interval,
      },
    }));
  };

  // Use this to inspect the brushedIntervals as they change!
  // console.log(
  //   JSON.stringify(brushedIntervals, null, 2),
  // );

  // Set up the SVG.
  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height)
    .style('background', '#090F10');

  var data = languages;
  // Invoke the parallel coordinates.
  svg.call(parallelCoordinates, {
    data,
    columns,
    columnTypes,
    colorValue,
    idValue,
    width,
    height,
    brushedIntervals,
    updateBrushedInterval,
  });
}